package edu.columbia.sip.buddy;

import edu.columbia.sip.AvailabilityEvent;
import edu.columbia.sip.AvailabilityListener;
import edu.columbia.sip.SIPConnection;
import edu.columbia.sip.SIPMessage;
import edu.columbia.sip.SIPRequest;
import edu.columbia.sip.SIPServer;
import edu.columbia.sip.SIPToken;
import edu.columbia.sip.SIPURL;

import com.sun.java.swing.BorderFactory;
import com.sun.java.swing.ImageIcon;
import com.sun.java.swing.JButton;
import com.sun.java.swing.JFrame;
import com.sun.java.swing.JLabel;
import com.sun.java.swing.JPanel;
import com.sun.java.swing.JPopupMenu;
import com.sun.java.swing.JTextArea;
import com.sun.java.swing.JScrollPane;
import com.sun.java.swing.JTabbedPane;
import com.sun.java.swing.UIManager;

import com.sun.java.swing.border.Border;

import com.sun.java.swing.event.ChangeEvent;
import com.sun.java.swing.event.ChangeListener;

import java.awt.Color;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import mjs.graphical.jfc.ActiveButton;

public class BuddyListApp extends JFrame {
    private static SIPServer _server;

    private static BuddyList _actives;
    private static BuddyList _buddies;
    private static SelectionManager _selectionManager;
    private static SIPURL _url;
    private static String _password;
    
    /** address of the SIP server with which to register */
    private static String _registrar;
    
    private JTabbedPane _tabs;
    private Color activeTabBG;
    private static JFrame _topLevelFrame;
    private JPopupMenu _popupMenu;
    private ActiveButton config;
    private JPanel bottombar;
        
    
    public BuddyListApp() {
        // create the components with which we'll interact 
        _selectionManager = new SelectionManager();
        
        // set up a user agent to listen to SIP messages for us
        try {
            _server = new SIPServer(SIPServer.USER_AGENT, 0);
            _server.start();
        } catch (java.io.IOException ioe) {
            System.out.println("BuddyListApp error during setup: unable to run a " +
            "user agent: \n" + ioe);
            System.exit(1);
        }

        try {
            // register with the system SIP Server and tell it to send
            // SIP messages to our user agent.
            SIPRequest register = new SIPRequest(SIPToken.REGISTER, _url, _server.getURL());
            register.respondToServer(_server);
            register.setHeader(SIPToken.TO, _registrar);
            register.setHeader(SIPToken.EXPIRES, "7200");
            SIPConnection conn = new SIPConnection(_registrar, SIPServer.DEFAULT_PORT);
            conn.putMessage(register);
            conn.close();
        } catch (Exception e) {
            System.out.println("BuddyListApp error during setup: cannot register " +
                                "with system SIP server: " + e);
            System.out.println("Other users may not be able to locate this agent\n" +
                                "without a server running on the default port.");
        }
        // ignore response to registration
        
        // later we'll add our buddy objects to our user agent as 
        // AvailabilityListeners
        
        
        // put together the GUI
        _topLevelFrame = this;
        _actives = new BuddyList();
        _actives.setMultipleMode(true);
        // here's where we might allow for a serialized list of buddies...
        _buddies = new BuddyList();
//        _buddies.add(new Buddy("Jude"));
//       _buddies.add(new Buddy("Tim"));
//        Buddy mom = new Buddy("Mom");
//        mom.setName("Sima Sambol");
//        mom.setSIPURL("sip://ssambol@netcom.com");
//        _buddies.add(mom);
//        _buddies.add(new Buddy("Mike"));
//        _buddies.add(new Buddy("Susan"));
//        _buddies.add(new Buddy("Jessy"));
//        _buddies.add(new Buddy("Lisa"));
        
        Color buttonBG = new Color(20, 20, 100);
        activeTabBG = new Color(0, 100, 50);
        
        Container c = getContentPane();
        setTitle("SIP Buddy List System: " + _url.toString());
        GridBagLayout gridbag = new GridBagLayout();
        c.setLayout(gridbag);
        GridBagConstraints constr = new GridBagConstraints();

        // set up top of main window with image button
        ImageIcon logo = new ImageIcon("buddylist.gif");
        JButton home = new JButton(logo);
        home.setBorder(BorderFactory.createRaisedBevelBorder());

        home.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                // open netscape to our home page
                Runtime rt = Runtime.getRuntime();
                String platform = System.getProperty("os.name");
                String netscape;
                if (platform.startsWith("Windows")) {
                    netscape = "C:\\program files\\netscape\\communicator\\program\\netscape";
                } else {
                    netscape = "netscape";
                }
                try {
                    rt.exec(netscape + " http://www.cs.columbia.edu/~mjs/ais");
                } catch (java.io.IOException ioe) {
                    System.out.println("Got IOE: " + ioe);
                }
            }
        });

        constr.gridwidth = constr.REMAINDER;
        constr.insets = new Insets(2, 0, 2, 0); // top, left, bot, right
        gridbag.setConstraints(home, constr);
        c.add (home);

        // set up center of main window with list tabs
        _tabs = new JTabbedPane();

        // add tabs: 1 for online active list, 2 for list configuration
        JPanel activePanel = new JPanel();
        GridBagLayout activeLayout = new GridBagLayout();
        GridBagConstraints aconst = new GridBagConstraints();
        activePanel.setLayout(activeLayout);
        
        // constraints for layout...
        aconst.gridwidth = aconst.REMAINDER;
        aconst.fill = aconst.BOTH;
        aconst.weightx = 1;
        aconst.weighty = 1;
        activeLayout.setConstraints(_actives, aconst);
        activePanel.add(_actives);
        
        JPanel buddiesPanel = new JPanel();
        GridBagLayout gbl = new GridBagLayout();
        buddiesPanel.setLayout(gbl);
        gbl.setConstraints(_buddies, aconst);
        buddiesPanel.add(_buddies);
        
        // button bar at the bottom of the non-active buddies tab
        JPanel bbb = new BuddiesButtonBar(_buddies);
        buddiesPanel.add(bbb);
    
        // an action listener for when one of the buddies is double clicked
        _buddies.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                BuddyEditor be = new BuddyEditor(_buddies.getBuddyAt(
                                                    _buddies.getSelectedIndex()));
                be.show();                                    
            }
        });
        
        _actives.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                SessionEditor se = new SessionEditor();
                se.show();
            }
        });

        _actives.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent ie) {
                Object oa[] = ie.getItemSelectable().getSelectedObjects();
                SelectionManager sm = BuddyListApp.getSelectionManager();
                sm.setSelection(oa);
            }
        });
        
        _tabs.addTab("Online", new JScrollPane(activePanel));
        _tabs.addTab("List Setup", new JScrollPane(buddiesPanel));
        _tabs.setSelectedIndex(0);
        _tabs.setForegroundAt(0, activeTabBG);
        _tabs.setOpaque(true);
        
        // this listener will change the color of the text of the active tab (just the 
        // label on the tab) when the user switches tabs.
        _tabs.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                int curr = _tabs.getSelectedIndex();
                _tabs.setForegroundAt(curr, activeTabBG);
                int other = (curr == 1) ? 0 : 1;
                _tabs.setForegroundAt(other, null);
            }
        });
                
        // lay out the tab panel in the main window
        constr = new GridBagConstraints();
        constr.weighty = 1.0;
        constr.weightx = 1.0;
        constr.fill = constr.BOTH;
        constr.gridwidth = constr.REMAINDER;
        gridbag.setConstraints(_tabs, constr);
        c.add(_tabs);

        // create a button bar to appear beneath the tab panel
        bottombar = new JPanel();
        config = new ActiveButton("Configure", "Configure", Color.white, Color.cyan,
                                    buttonBG, buttonBG, null, null);
        config.setHighlightState(false);
        _popupMenu = new MainConfigMenu();
        _popupMenu.setInvoker(config);
        config.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                _popupMenu.show(BuddyListApp.getTopLevelFrame(),
                                bottombar.getX() + config.getX(), 
                                bottombar.getY() + config.getY() - 40);
            }
        });
        
        bottombar.add(config);
        // a listener for the quit button
        ActiveButton quit = new ActiveButton("Quit", "Quit", Color.white, Color.red, 
                                            buttonBG, buttonBG, null, null);
        quit.setHighlightState(false);                                    
        quit.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                // first unregister this user from wherever he/she is registered as
                // available
                try {
                    // register with the system SIP Server and tell it to send
                    // SIP messages to our user agent.
                    SIPRequest unregister = new SIPRequest(SIPToken.REGISTER, _url, _server.getURL());
                    unregister.respondToServer(_server);
                    unregister.setHeader(SIPToken.TO, _registrar);
                    unregister.setHeader(SIPToken.EXPIRES, "0");
                    SIPConnection conn = new SIPConnection(_registrar, SIPServer.DEFAULT_PORT);
                    conn.putMessage(unregister);
                    conn.close();
                } catch (Exception e) {
                    System.out.println("BuddyListApp error during signoff: cannot unregister " +
                                        "with system SIP server: " + e);
                }
                // ignore response to unregistration

                // now quit the program 
                System.exit(0);
            }
        });
        
        // make sure that the window controls actually work properly
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
        
        bottombar.add(quit);
        c.add(bottombar);
        
        // give the application an initial size 
        setSize(275, 450);
    }

    public static BuddyList getActiveList() {
        return _actives;
    }
    
    public static SIPServer getServer() {
        return _server;
    }
        
    public static SelectionManager getSelectionManager() {
        return _selectionManager;
    }
    
    public static void main(String args[]) {
        if (args.length !=2 ) {
            usage();
        }

        _url = new SIPURL(args[0]);
        _password = "NoPasswordYet"; //args[1];
        
        _registrar = args[1];
        
        /* commenting out to make usage more simple.  
        
        // let the user select the look and feel - either Metal or the local l&f
        if (args[3].charAt(0) != 'M') {        */
            try { 
                UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName() );
            } catch( Exception e ) {
                System.err.println( "Could not load look and feel" );
                e.printStackTrace();
            }
       // }

        BuddyListApp bla = new BuddyListApp();
        bla.show();
    }
    
    public static JFrame getTopLevelFrame() {
        return _topLevelFrame;
    }

    public static BuddyList getBuddies() {
        return _buddies;
    }
    
    public static void usage() {
        System.out.println("Usage: " +
        "java edu.columbia.sip.BuddyListApp <SIP URL> <registrar's  IP address>");
    //    System.out.println("where lookandfeel is 'M' for Metal or 'L' for local L&F.");
        System.exit(1);
    }

}